@@ -21,6 +21,8 @@ static char sccsid[] = "@(#)gethostnamadr.c	6.39 (Berkeley) 1/4/90";
 
 #include "inetprivate.h"
 #include <syslog.h>
+#include <errno.h>
+#include <netdb.h>
 
 #if defined(USE_OPTIONS_H)
 # include "res_options.h"
@@ -121,13 +123,6 @@ typedef union {
     char ac;
 } align;
 
-#ifdef SHLIB
-extern int h_errno;
-#else
-extern int h_errno;
-#endif
-extern int errno;
-
 #if NLS
 #include "nl_types.h"
 #endif
@@ -335,7 +330,7 @@ init_services()
 
 			for (cp=buf; cp && *cp && isspace(*cp); cp++);
 			if  (cp && *cp) {
-			strcpy(buf,cp);
+			memmove(buf, cp, strlen(cp) + 1);
 			if (checkbuf(buf, CMD_ORDER)) {
 				cp = strpbrk(buf, " \t");
 				if (!cp 
@@ -352,9 +347,11 @@ init_services()
 						cp++;
 					dp = strpbrk(cp, TOKEN_SEPARATORS);
 					if (dp) *dp = '\0';
-					if (checkbuf(cp, ORD_BIND))
+					if (checkbuf(cp, ORD_BIND)) {
 						service_order[cc++] = SERVICE_BIND;
-					else if (checkbuf(cp, ORD_HOSTS))
+						if (!(_res.options & RES_INIT))
+							res_init();
+					} else if (checkbuf(cp, ORD_HOSTS))
 						service_order[cc++] = SERVICE_HOSTS;
 					else if (checkbuf(cp, ORD_NIS))
 						service_order[cc++] = SERVICE_NIS;
@@ -478,7 +475,7 @@ init_services()
                                     if ( cp = strpbrk(buf, " \t") )
                                     {
 					while (*cp == ' ' || *cp == '\t') cp++;
-                                        if (cp)
+                                        if (*cp)
                                         {
                                             (void) strcpy(tdp,cp);
                                             trimdomain[numtrimdomains++]=tdp;
@@ -511,9 +508,11 @@ init_services()
 		cc=0;
 		if(NULL!=(cp=strtok(cp, TOKEN_SEPARATORS))){
 			do{
-				if(checkbuf(cp, ORD_BIND))
+				if(checkbuf(cp, ORD_BIND)) {
 					service_order[cc++] = SERVICE_BIND;
-				else if (checkbuf(cp, ORD_HOSTS))
+					if (!(_res.options & RES_INIT))
+						res_init();
+				} else if (checkbuf(cp, ORD_HOSTS))
 					service_order[cc++] = SERVICE_HOSTS;
 				else if (checkbuf(cp, ORD_NIS))
 					service_order[cc++] = SERVICE_NIS;
@@ -580,8 +579,6 @@ init_services()
 	}
 
 	service_done = 1;
-	if (!(_res.options & RES_INIT))
-		res_init();
 }
 
 static struct hostent *
@@ -860,7 +857,10 @@ gethostent ()
 	if (h_addr_ptrs[1] && reorder)
 	  reorder_addrs(hp);
 	if (hp)
+	{
+	  h_errno = NETDB_SUCCESS;
 	  return hp;
+	}
 	h_errno = HOST_NOT_FOUND;
 	break;
 
@@ -870,7 +870,10 @@ gethostent ()
 	if (h_addr_ptrs[1] && reorder)
 	  reorder_addrs(hp);
 	if (hp)
+	{
+	  h_errno = NETDB_SUCCESS;
 	  return hp;
+	}
 	h_errno = HOST_NOT_FOUND;
 	break;
 #endif /* NIS */
@@ -942,28 +945,55 @@ gethostbyname(const char *name)
 				if (_res.options & RES_DEBUG)
 					printf("res_search failed\n");
 #endif
+				break;
 			}
 			hp = getanswer(&buf, n, name, C_IN, T_A);
 			if (h_addr_ptrs[1] && reorder)
 				reorder_addrs(hp);
 			if (hp)
+			{
+				h_errno = NETDB_SUCCESS;
 				return trim_domains(hp);
+			}
 			break;
 		case SERVICE_HOSTS:
-			hp = _gethtbyname(name);
+			if (*trimdomain) {			
+				char *p;
+				p = strdup(name);
+				dotrimdomain(p);
+				hp = _gethtbyname(p);
+				free(p);
+			}
+			else
+			{
+				hp = _gethtbyname(name);
+			}
 			if (h_addr_ptrs[1] && reorder)
 				reorder_addrs(hp);
 			if (hp)
-				return hp;
+			{
+				h_errno = NETDB_SUCCESS;
+				return trim_domains(hp);
+			}
 			h_errno = HOST_NOT_FOUND;
 			break;
 #ifdef NIS
 		case SERVICE_NIS:
-			hp = _getnishost(name, "hosts.byname");
+			if (*trimdomain) {			
+				char *p;
+				p = strdup(name);
+				dotrimdomain(p);
+				hp = _getnishost(p, "hosts.byname");
+				free(p);
+			} else
+				hp = _getnishost(name, "hosts.byname");
 			if (h_addr_ptrs[1] && reorder)
 				reorder_addrs(hp);
 			if (hp)
-				return hp;
+			{
+				h_errno = NETDB_SUCCESS;
+				return trim_domains(hp);
+			}
 			h_errno = HOST_NOT_FOUND;
 			break;
 #endif /* NIS */
@@ -1057,11 +1087,17 @@ gethostbyaddr(const char *addr, int len, int type)
 					for (addrs = hp->h_addr_list;
 						*addrs; addrs++){
 						if (!bcmp(addrs[0], addr, len))
+						{
+							h_errno = NETDB_SUCCESS;
 							return trim_domains(hp);
+						}
 					}
 #else
 					if (!bcmp(hp->h_addr, addr, len)))
+					{
+						h_errno = NETDB_SUCCESS;
 						return trim_domains(hp);
+					}
 #endif
 					/* We've been spoofed */
 					h_errno = HOST_NOT_FOUND;
@@ -1092,6 +1128,7 @@ gethostbyaddr(const char *addr, int len, int type)
 #if BSD < 43 && !defined(h_addr)	/* new-style hostent structure */
 				hp->h_addr = h_addr_ptrs[0];
 #endif
+				h_errno = NETDB_SUCCESS;
 				return trim_domains(hp);
 			}
 			h_errno = HOST_NOT_FOUND;
@@ -1099,7 +1136,10 @@ gethostbyaddr(const char *addr, int len, int type)
 		case SERVICE_HOSTS:
 			hp = _gethtbyaddr(addr, len, type);
 			if (hp)
+			{
+				h_errno = NETDB_SUCCESS;
 				return hp;
+			}
 			h_errno = HOST_NOT_FOUND;
 			break;
 #ifdef NIS
@@ -1111,7 +1151,10 @@ gethostbyaddr(const char *addr, int len, int type)
 				      ((unsigned)addr[3] & 0xff));
 			hp = _getnishost(qbuf, "hosts.byaddr");
 			if (hp)
+			{
+				h_errno = NETDB_SUCCESS;
 				return hp;
+			}
 			h_errno = HOST_NOT_FOUND;
 			break;
 #endif /* NIS */
