@@ -55,7 +55,7 @@
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static char sccsid[] = "@(#)res_send.c	8.1 (Berkeley) 6/4/93";
-static char rcsid[] = "res_send.c,v 1.2 1995/05/06 14:12:44 hjl Exp";
+static char rcsid[] = "res_send.c,v 1.3 1995/10/15 15:10:35 hjl Exp";
 #endif /* LIBC_SCCS and not lint */
 
 	/* change this to "0"
@@ -78,10 +78,12 @@ static char rcsid[] = "res_send.c,v 1.2 1995/05/06 14:12:44 hjl Exp";
 #include <arpa/inet.h>
 
 #include <stdio.h>
+#include <netdb.h>
 #include <errno.h>
 #include <resolv.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "res_options.h"
 
@@ -101,16 +103,23 @@ static int vc = 0;	/* is the socket a virtual ciruit? */
 #define FD_ZERO(p)	bzero((char *)(p), sizeof(*(p)))
 #endif
 
+/* XXX - this should be done in portability.h */
+#if (defined(BSD) && (BSD >= 199103)) || defined(linux)
+# define CAN_RECONNECT 1
+#else
+# define CAN_RECONNECT 0
+#endif
+
 #ifndef DEBUG
 #   define Dprint(cond, args) /*empty*/
-#   define DprintQ(cond, args, query) /*empty*/
+#   define DprintQ(cond, args, query, size) /*empty*/
 #   define Aerror(file, string, error, address) /*empty*/
 #   define Perror(file, string, error) /*empty*/
 #else
 #   define Dprint(cond, args) if (cond) {fprintf args;} else {}
-#   define DprintQ(cond, args, query) if (cond) {\
+#   define DprintQ(cond, args, query, size) if (cond) {\
 			fprintf args;\
-			__p_query(query);\
+			__fp_nquery(query, size, stdout);\
 		} else {}
     static void
     Aerror(file, string, error, address)
@@ -224,7 +233,7 @@ res_nameinquery(name, type, class, buf, eom)
 		if (n < 0)
 			return (-1);
 		cp += n;
-		ttype = _getshort(cp);	cp += INT16SZ;
+		ttype = _getshort(cp); cp += INT16SZ;
 		tclass = _getshort(cp); cp += INT16SZ;
 		if (ttype == type &&
 		    tclass == class &&
@@ -284,10 +293,12 @@ res_send(buf, buflen, ans, anssiz)
 	register int n;
 	u_int badns;	/* XXX NSMAX can't exceed #/bits in this var */
 
-	DprintQ((_res.options & RES_DEBUG) || (_res.pfcode & RES_PRF_QUERY),
-		(stdout, ";; res_send()\n"), buf);
-	if (!(_res.options & RES_INIT) && res_init() == -1)
+	if ((_res.options & RES_INIT) == 0 && res_init() == -1) {
+		/* errno should have been set by res_init() in this case. */
 		return (-1);
+	}
+	DprintQ((_res.options & RES_DEBUG) || (_res.pfcode & RES_PRF_QUERY),
+		(stdout, ";; res_send()\n"), buf, buflen);
 	v_circuit = (_res.options & RES_USEVC) || buflen > PACKETSZ;
 	gotsomewhere = 0;
 	connreset = 0;
@@ -356,12 +367,13 @@ res_send(buf, buflen, ans, anssiz)
 				if (s >= 0)
 					_res_close();
 
-				s = socket(AF_INET, SOCK_STREAM, PF_UNSPEC);
+				s = socket(PF_INET, SOCK_STREAM, 0);
 				if (s < 0) {
 					terrno = errno;
 					Perror(stderr, "socket(vc)", errno);
 					return (-1);
 				}
+				errno = 0;
 				if (connect(s, (struct sockaddr *)nsap,
 					    sizeof(struct sockaddr)) < 0) {
 					terrno = errno;
@@ -471,9 +483,12 @@ res_send(buf, buflen, ans, anssiz)
 			if ((s < 0) || vc) {
 				if (vc)
 					_res_close();
-				s = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC);
+				s = socket(PF_INET, SOCK_DGRAM, 0);
 				if (s < 0) {
- bad_dg_sock:				terrno = errno;
+#if !CAN_RECONNECT
+ bad_dg_sock:
+#endif
+					terrno = errno;
 					Perror(stderr, "socket(dg)", errno);
 					return (-1);
 				}
@@ -524,7 +539,7 @@ res_send(buf, buflen, ans, anssiz)
 				 * for responses from more than one server.
 				 */
 				if (connected) {
-#if defined(BSD) && (BSD >= 199103)
+#if CAN_RECONNECT
 					struct sockaddr_in no_addr;
 
 					no_addr.sin_family = AF_INET;
@@ -535,8 +550,7 @@ res_send(buf, buflen, ans, anssiz)
 						        &no_addr,
 						       sizeof(no_addr));
 #else
-					int s1 = socket(AF_INET, SOCK_DGRAM,
-							PF_UNSPEC);
+					int s1 = socket(PF_INET, SOCK_DGRAM,0);
 					if (s1 < 0)
 						goto bad_dg_sock;
 					(void) dup2(s1, s);
@@ -587,6 +601,7 @@ res_send(buf, buflen, ans, anssiz)
 				_res_close();
 				goto next_ns;
 			}
+			errno = 0;
 			fromlen = sizeof(struct sockaddr_in);
 			resplen = recvfrom(s, (char*)ans, anssiz, 0,
 					   (struct sockaddr *)&from, &fromlen);
@@ -605,7 +620,7 @@ res_send(buf, buflen, ans, anssiz)
 				DprintQ((_res.options & RES_DEBUG) ||
 					(_res.pfcode & RES_PRF_REPLY),
 					(stdout, ";; old answer:\n"),
-					ans);
+					ans, resplen);
 				goto wait;
 			}
 #if CHECK_SRVR_ADDR
@@ -619,7 +634,7 @@ res_send(buf, buflen, ans, anssiz)
 				DprintQ((_res.options & RES_DEBUG) ||
 					(_res.pfcode & RES_PRF_REPLY),
 					(stdout, ";; not our server:\n"),
-					ans);
+					ans, resplen);
 				goto wait;
 			}
 #endif
@@ -634,7 +649,7 @@ res_send(buf, buflen, ans, anssiz)
 				DprintQ((_res.options & RES_DEBUG) ||
 					(_res.pfcode & RES_PRF_REPLY),
 					(stdout, ";; wrong query name:\n"),
-					ans);
+					ans, resplen);
 				goto wait;
 			}
 			if (anhp->rcode == SERVFAIL ||
@@ -642,7 +657,7 @@ res_send(buf, buflen, ans, anssiz)
 			    anhp->rcode == REFUSED) {
 				DprintQ(_res.options & RES_DEBUG,
 					(stdout, "server rejected query:\n"),
-					ans);
+					ans, resplen);
 				badns |= (1 << ns);
 				_res_close();
 				/* don't retry if called from dig */
@@ -661,10 +676,14 @@ res_send(buf, buflen, ans, anssiz)
 				goto same_ns;
 			}
 		} /*if vc/dg*/
+		Dprint((_res.options & RES_DEBUG) ||
+		       ((_res.pfcode & RES_PRF_REPLY) &&
+			(_res.pfcode & RES_PRF_HEAD1)),
+		       (stdout, ";; got answer:\n"));
 		DprintQ((_res.options & RES_DEBUG) ||
 			(_res.pfcode & RES_PRF_REPLY),
-			(stdout, ";; got answer:\n"),
-			ans);
+			(stdout, ""),
+			ans, resplen);
 		/*
 		 * If using virtual circuits, we assume that the first server
 		 * is preferred over the rest (i.e. it is on the local
